import requests
from json import loads
from ..config import ConfigManager
from ..utils.system import createFallbackSecret, getFallbackSecret, pruneDictionary, getLocalSecret
from ..utils.crypto import aesDecryptSecret
from ..utils.url import url

def fetchSecrets(project, environment):
  configManager = ConfigManager()
  config = configManager.getConfig()
  query = {'query': 'query {generalPublicProjects(filterOptions: {title: "%s", disableCustomSelect: true}){list {id title publicEnvironments(filterOptions: {title: "%s"}){list {id key title}}}}}'%(project, environment)}
  try:
    response = requests.post(url, json=query, headers={'KEY': config["api_key"]})
    if 'errors' in response.json() and response.json()[0]['message'] == 'Unauthorized':
      print('Sorry you don\'t have access to this project environment any longer, please contact admin')
      return {'errors': 'Sorry you don\'t have access to this project environment any longer, please contact admin' }
    else:
      return response.json()
  except Exception:
    return {'errors': 'Sorry you don\'t have access to this project environment any longer, please contact admin' }


def downloadSecrets(project, environment):
  localSecret = getLocalSecret()
  finalEnvs = {**localSecret}
  try:
    secrets = fetchSecrets(project, environment)
    if 'errors' in secrets:
      fallbackSecrets = getFallbackSecret(project, environment)
      if 'errors' in fallbackSecrets:
        createdFallback = createFallbackSecret(project, finalEnvs, environment)
        if 'errors' in createdFallback:
          print('No fallback secret was found')
        return {'env': finalEnvs}
      else:
        print(pruneDictionary({**finalEnvs, **fallbackSecrets}))
        return {'env': pruneDictionary({**fallbackSecrets, **finalEnvs})}
    else:
      dataList = secrets['data']['generalPublicProjects']['list'][0]['publicEnvironments']['list']
      parsedDataList = loads(dataList[0]["key"])
      secretDict = {}
      for dic in parsedDataList:
        decrypted = aesDecryptSecret(dic)
        secretDict[decrypted['key']] = decrypted['value']
      mergedEnvs = pruneDictionary({**secretDict, **finalEnvs})
      createFallbackSecret(project, mergedEnvs, environment)
      return {'env': mergedEnvs}
  except Exception:
    print('There was an error fetching secrets for %s under the current organization. Reverting to fallback...'%(project))
    fallbackSecrets = getFallbackSecret(project, environment)
    if 'errors' in fallbackSecrets:
      createdFallback = createFallbackSecret(project, finalEnvs, environment)
      if 'errors' in createdFallback:
          print('No fallback secret was found')
      return {'env': finalEnvs}
    else:
      mergedEnvs = pruneDictionary({**fallbackSecrets, **finalEnvs})
      return {'env': mergedEnvs}



import os
import re
import subprocess
import platform
from Crypto.Hash import SHA256


class MachineID:

  system = platform.system()

  win32RegBinPath = { 'native': '%windir%\\System32',
        'mixed': '%windir%\\sysnative\\cmd.exe /c %windir%\\System32',
        '': ''}
  
  def isWindowsProcessMixedOrNativeArchitecture():
    if platform.system() != 'win32':
      return ''
    if platform.architecture() == 'ia32' and 'PROCESSOR_ARCHITEW6432' in os.environ:
      return 'mixed'
    return 'native'
  
  guid = {'darwin': 'ioreg -rd1 -c IOPlatformExpertDevice', 
          'win32': '%s\\REG.exe '%(win32RegBinPath[isWindowsProcessMixedOrNativeArchitecture()]) + 'QUERY HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Cryptography ' + '/v MachineGuid',
          'linux': '( cat /var/lib/dbus/machine-id /etc/machine-id 2> /dev/null || hostname ) | head -n 1 || :',
          'freebsd': 'kenv -q smbios.system.uuid || sysctl -n kern.hostuuid'}

  def hash(self, guid):
    return SHA256.new(data=guid.encode()).hexdigest()

  def expose(self, result):
    regex = r"\=|\s+|\""
    subst = ""
    system = self.system.lower()
    if(system == 'darwin'):   
      formattedResult = result.split('IOPlatformUUID')[1].split('\n')[0]
      return re.sub(regex, subst, formattedResult, 0, re.IGNORECASE).lower()
    elif(system == 'win32'):
      formattedResult = str(result).split('REG_SZ')[1]
      return re.sub(regex, subst, formattedResult, 0, re.IGNORECASE).lower()
    elif(system == 'linux'):
      formattedResult = str(result)
      return re.sub(regex, subst, formattedResult, 0, re.IGNORECASE).lower()
    elif(system == 'freebsd'):
      formattedResult = str(result)
      return re.sub(regex, subst, formattedResult, 0, re.IGNORECASE).lower()
    else:
      raise Exception('Unsupported platform: %s'%(system))
    
  def getMachineId(self, original = False):
    process = subprocess.Popen(self.guid[self.system.lower()].split(), stdout=subprocess.PIPE)
    output, error = process.communicate()
    if error:
      raise Exception('Error while obtaining machine id %s'%(error))
    id = self.expose(output.decode())
    return (self.hash(id), id) [original]
    


from os.path import join, expanduser, exists
from os import mkdir
from json import loads
from ..utils.crypto import decryptFallback, encrypt
from ..utils.machineid import MachineID
from ..config import ConfigManager

def getLocalSecret():
  configManager = ConfigManager()
  config = configManager.getConfig()
  try: 
    localSecrets = config["secrets"]["local"] or {}
    return parsedLocalSecret(localSecrets)
  except Exception:
    return {}

def parsedLocalSecret(secrets):
  secretStore = {}
  if isinstance(secrets, list): 
    for secret_dict in secrets:
      secretStore[next(iter(secret_dict))] = secret_dict[next(iter(secret_dict))]
  else:
    for secret in secrets.keys():
      secretStore[secret] = secrets[secret]
  return secretStore

def getHomeDirectory():
  return expanduser('~')

def getFallbackDirectory(project):
  fallbackDirectory = join(getHomeDirectory(), '.onboardbase', 'fallback', project)
  if not exists(fallbackDirectory):
    mkdir(fallbackDirectory)
  return fallbackDirectory

def getFallbackSecret(project, environment = 'development'):
  fallbackPath = join(getFallbackDirectory(project))
  fallbackEnvironmentPath = fallbackPath + '_%s'%(environment)
  machineid = MachineID()
  passphrase = machineid.getMachineId()
  if not exists(fallbackEnvironmentPath):
    print('Could not access project and no fallback exist for %s with the environment %s'%(project,environment))
    return {'errors': 'fallback secrets not yet saved'}
  fallbackFile = open(fallbackEnvironmentPath, 'rb')
  f = fallbackFile.read()
  decryptedSecret = decryptFallback(f, passphrase)
  decryptedSecret = decryptedSecret.replace("'", '"')
  return loads(decryptedSecret)

def createFallbackSecret(project, secrets, environment = 'development'):
  if len(secrets) == 0:
    return {'errors': 'Fallback secrets was not saved'}
  fallbackPath = join(getFallbackDirectory(project))
  fallbackEnvironmentPath = fallbackPath + '_%s'%(environment)
  encryptedSecrets = encrypt(secrets)
  f = open(fallbackEnvironmentPath, 'wb')
  f.write(encryptedSecrets)
  f.close()

def pruneDictionary(dictionary):
  temp = {val : key for key, val in dictionary.items()}
  res = {val : key for key, val in temp.items()}
  return res
  


import base64
import hashlib
from hashlib import md5
from json import loads
from Crypto import Random
from Crypto.Cipher import AES
from ..utils.machineid import MachineID
from ..config import ConfigManager


BLOCK_SIZE = 16

pad = lambda s: str(s) + (BLOCK_SIZE - len(str(s)) % BLOCK_SIZE) * chr(BLOCK_SIZE - len(str(s)) % BLOCK_SIZE)
unpad = lambda s: s[:-ord(s[len(s) - 1:])]
 
def encrypt(raw):
  machineid = MachineID()
  password = machineid.getMachineId()
  private_key = hashlib.sha256(password.encode("utf-8")).digest()
  raw = pad(raw)
  iv = Random.new().read(AES.block_size)
  cipher = AES.new(private_key, AES.MODE_CBC, iv)
  return base64.b64encode(iv + cipher.encrypt(raw.encode()))

 
def decryptFallback(enc, password):
    private_key = hashlib.sha256(password.encode("utf-8")).digest()
    enc = base64.b64decode(enc)
    iv = enc[:16]
    cipher = AES.new(private_key, AES.MODE_CBC, iv)
    return unpad(cipher.decrypt(enc[16:])).decode()

def bytes_to_key(data, salt, output=48):
    assert len(salt) == 8, len(salt)
    data += salt
    key = md5(data).digest()
    final_key = key
    while len(final_key) < output:
        key = md5(key + data).digest()
        final_key += key
    return final_key[:output]

def decryptRemote(encrypted, passphrase):
    encrypted = base64.b64decode(encrypted)
    assert encrypted[0:8] == b"Salted__"
    salt = encrypted[8:16]
    key_iv = bytes_to_key(passphrase, salt, 32+16)
    key = key_iv[:32]
    iv = key_iv[32:]
    aes = AES.new(key, AES.MODE_CBC, iv)
    return unpad(aes.decrypt(encrypted[16:])).decode()

def aesDecryptSecret(secret):
  configManager = ConfigManager()
  config = configManager.getConfig()
  json_data = decryptRemote(secret, config["passcode"].encode())
  json_data = loads(json_data)
  return json_data